# Auto generated Python source file from Ada sources
# Call 'make' in 'src/vhdl' to regenerate:
#
from pydecor import export


@export
class Tok:
    Invalid = 0
    Eof = 1
    Newline = 2
    Block_Comment_Start = 3
    Block_Comment_End = 4
    Block_Comment_Text = 5
    Line_Comment = 6
    Character = 7
    Identifier = 8
    Integer = 9
    Real = 10
    String = 11
    Bit_String = 12
    Integer_Letter = 13
    Left_Paren = 14
    Right_Paren = 15
    Left_Bracket = 16
    Right_Bracket = 17
    Colon = 18
    Semi_Colon = 19
    Comma = 20
    Double_Arrow = 21
    Tick = 22
    Double_Star = 23
    Assign = 24
    Bar = 25
    Box = 26
    Dot = 27
    Equal_Equal = 28
    Equal = 29
    Not_Equal = 30
    Less = 31
    Less_Equal = 32
    Greater = 33
    Greater_Equal = 34
    Match_Equal = 35
    Match_Not_Equal = 36
    Match_Less = 37
    Match_Less_Equal = 38
    Match_Greater = 39
    Match_Greater_Equal = 40
    Plus = 41
    Minus = 42
    Ampersand = 43
    Condition = 44
    Double_Less = 45
    Double_Greater = 46
    Caret = 47
    And_And = 48
    Bar_Bar = 49
    Left_Curly = 50
    Right_Curly = 51
    Exclam_Mark = 52
    Brack_Star = 53
    Brack_Plus_Brack = 54
    Brack_Arrow = 55
    Brack_Equal = 56
    Bar_Arrow = 57
    Bar_Double_Arrow = 58
    Minus_Greater = 59
    Equiv_Arrow = 60
    Arobase = 61
    Star = 62
    Slash = 63
    Mod = 64
    Rem = 65
    Abs = 66
    Not = 67
    Access = 68
    After = 69
    Alias = 70
    All = 71
    Architecture = 72
    Array = 73
    Assert = 74
    Attribute = 75
    Begin = 76
    Block = 77
    Body = 78
    Buffer = 79
    Bus = 80
    Case = 81
    Component = 82
    Configuration = 83
    Constant = 84
    Disconnect = 85
    Downto = 86
    Else = 87
    Elsif = 88
    End = 89
    Entity = 90
    Exit = 91
    File = 92
    For = 93
    Function = 94
    Generate = 95
    Generic = 96
    Guarded = 97
    If = 98
    In = 99
    Inout = 100
    Is = 101
    Label = 102
    Library = 103
    Linkage = 104
    Loop = 105
    Map = 106
    New = 107
    Next = 108
    Null = 109
    Of = 110
    On = 111
    Open = 112
    Others = 113
    Out = 114
    Package = 115
    Port = 116
    Procedure = 117
    Process = 118
    Range = 119
    Record = 120
    Register = 121
    Report = 122
    Return = 123
    Select = 124
    Severity = 125
    Signal = 126
    Subtype = 127
    Then = 128
    To = 129
    Transport = 130
    Type = 131
    Units = 132
    Until = 133
    Use = 134
    Variable = 135
    Wait = 136
    When = 137
    While = 138
    With = 139
    And = 140
    Or = 141
    Xor = 142
    Nand = 143
    Nor = 144
    Xnor = 145
    Group = 146
    Impure = 147
    Inertial = 148
    Literal = 149
    Postponed = 150
    Pure = 151
    Reject = 152
    Shared = 153
    Unaffected = 154
    Sll = 155
    Sla = 156
    Sra = 157
    Srl = 158
    Rol = 159
    Ror = 160
    Protected = 161
    Assume = 162
    Context = 163
    Cover = 164
    Default = 165
    Force = 166
    Parameter = 167
    Property = 168
    Release = 169
    Restrict = 170
    Restrict_Guarantee = 171
    Sequence = 172
    Vmode = 173
    Vprop = 174
    Vunit = 175
    Across = 176
    Break = 177
    Limit = 178
    Nature = 179
    Noise = 180
    Procedural = 181
    Quantity = 182
    Reference = 183
    Spectrum = 184
    Subnature = 185
    Terminal = 186
    Through = 187
    Tolerance = 188
    Psl_Clock = 189
    Psl_Endpoint = 190
    Psl_Const = 191
    Psl_Boolean = 192
    Inf = 193
    Within = 194
    Abort = 195
    Before = 196
    Before_Em = 197
    Before_Un = 198
    Before_Em_Un = 199
    Always = 200
    Never = 201
    Eventually_Em = 202
    Next_Em = 203
    Next_A = 204
    Next_A_Em = 205
    Next_E = 206
    Next_E_Em = 207
    Next_Event = 208
    Next_Event_Em = 209
    Next_Event_A = 210
    Next_Event_A_Em = 211
    Next_Event_E = 212
    Next_Event_E_Em = 213
    Until_Em = 214
    Until_Un = 215
    Until_Em_Un = 216
    Prev = 217
    Stable = 218
    Fell = 219
    Rose = 220
