# Auto generated Python source file from Ada sources
# Call 'make' in 'src/vhdl' to regenerate:
#
from pydecor import export


@export
class Name:
    First_Character = 1
    Last_Character = 256
    First_Keyword = 257
    Mod = 257
    Rem = 258
    Abs = 259
    Not = 260
    Access = 261
    After = 262
    Alias = 263
    All = 264
    Architecture = 265
    Array = 266
    Assert = 267
    Attribute = 268
    Begin = 269
    Block = 270
    Body = 271
    Buffer = 272
    Bus = 273
    Case = 274
    Component = 275
    Configuration = 276
    Constant = 277
    Disconnect = 278
    Downto = 279
    Else = 280
    Elsif = 281
    End = 282
    Entity = 283
    Exit = 284
    File = 285
    For = 286
    Function = 287
    Generate = 288
    Generic = 289
    Guarded = 290
    If = 291
    In = 292
    Inout = 293
    Is = 294
    Label = 295
    Library = 296
    Linkage = 297
    Loop = 298
    Map = 299
    New = 300
    Next = 301
    Null = 302
    Of = 303
    On = 304
    Open = 305
    Others = 306
    Out = 307
    Package = 308
    Port = 309
    Procedure = 310
    Process = 311
    Range = 312
    Record = 313
    Register = 314
    Report = 315
    Return = 316
    Select = 317
    Severity = 318
    Signal = 319
    Subtype = 320
    Then = 321
    To = 322
    Transport = 323
    Type = 324
    Units = 325
    Until = 326
    Use = 327
    Variable = 328
    Wait = 329
    When = 330
    While = 331
    With = 332
    And = 333
    Or = 334
    Xor = 335
    Nand = 336
    Nor = 337
    Last_Vhdl87 = 337
    Xnor = 338
    Group = 339
    Impure = 340
    Inertial = 341
    Literal = 342
    Postponed = 343
    Pure = 344
    Reject = 345
    Shared = 346
    Unaffected = 347
    Sll = 348
    Sla = 349
    Sra = 350
    Srl = 351
    Rol = 352
    Ror = 353
    Last_Vhdl93 = 353
    Protected = 354
    Last_Vhdl00 = 354
    Assume = 355
    Context = 356
    Cover = 357
    Default = 358
    Force = 359
    Parameter = 360
    Property = 361
    Release = 362
    Restrict = 363
    Restrict_Guarantee = 364
    Sequence = 365
    Vmode = 366
    Vprop = 367
    Vunit = 368
    Last_Vhdl08 = 368
    First_Ams_Keyword = 369
    Across = 369
    Break = 370
    Limit = 371
    Nature = 372
    Noise = 373
    Procedural = 374
    Quantity = 375
    Reference = 376
    Spectrum = 377
    Subnature = 378
    Terminal = 379
    Through = 380
    Tolerance = 381
    Last_AMS_Vhdl = 381
    Last_Keyword = 381
    First_Verilog = 382
    Always = 382
    Assign = 383
    Buf = 384
    Bufif0 = 385
    Bufif1 = 386
    Casex = 387
    Casez = 388
    Cmos = 389
    Deassign = 390
    Defparam = 391
    Disable = 392
    Edge = 393
    Endcase = 394
    Endfunction = 395
    Endmodule = 396
    Endprimitive = 397
    Endspecify = 398
    Endtable = 399
    Endtask = 400
    Forever = 401
    Fork = 402
    Highz0 = 403
    Highz1 = 404
    Ifnone = 405
    Initial = 406
    Input = 407
    Join = 408
    Large = 409
    Macromodule = 410
    Medium = 411
    Module = 412
    Negedge = 413
    Nmos = 414
    Notif0 = 415
    Notif1 = 416
    Output = 417
    Pmos = 418
    Posedge = 419
    Primitive = 420
    Pull0 = 421
    Pull1 = 422
    Pulldown = 423
    Pullup = 424
    Realtime = 425
    Reg = 426
    Repeat = 427
    Rcmos = 428
    Rnmos = 429
    Rpmos = 430
    Rtran = 431
    Rtranif0 = 432
    Rtranif1 = 433
    Scalared = 434
    Small = 435
    Specify = 436
    Specparam = 437
    Strong0 = 438
    Strong1 = 439
    Supply0 = 440
    Supply1 = 441
    Tablex = 442
    Task = 443
    Tran = 444
    Tranif0 = 445
    Tranif1 = 446
    Tri = 447
    Tri0 = 448
    Tri1 = 449
    Triand = 450
    Trior = 451
    Trireg = 452
    Vectored = 453
    Wand = 454
    Weak0 = 455
    Weak1 = 456
    Wire = 457
    Wor = 458
    Last_Verilog = 458
    First_V2001 = 459
    Automatic = 459
    Endgenerate = 460
    Genvar = 461
    Localparam = 462
    Unsigned = 463
    Signed = 464
    Last_V2001 = 464
    Uwire = 465
    First_SV3_0 = 466
    Always_Comb = 466
    Always_Ff = 467
    Always_Latch = 468
    Bit = 469
    Byte = 470
    Changed = 471
    Char = 472
    Const = 473
    Continue = 474
    Do = 475
    Endinterface = 476
    Endtransition = 477
    Enum = 478
    Export = 479
    Extern = 480
    Forkjoin = 481
    Iff = 482
    Import = 483
    Int = 484
    Interface = 485
    Logic = 486
    Longint = 487
    Longreal = 488
    Modport = 489
    Packed = 490
    Priority = 491
    Shortint = 492
    Shortreal = 493
    Static = 494
    Struct = 495
    Timeprecision = 496
    Timeunit = 497
    Transition = 498
    Typedef = 499
    Union = 500
    Unique = 501
    Unique0 = 502
    Void = 503
    Last_SV3_0 = 503
    First_SV3_1 = 504
    Chandle = 504
    Class = 505
    Clocking = 506
    Constraint = 507
    Dist = 508
    Endclass = 509
    Endclocking = 510
    Endprogram = 511
    Endproperty = 512
    Endsequence = 513
    Extends = 514
    Final = 515
    First_Match = 516
    Inside = 517
    Intersect = 518
    Join_Any = 519
    Join_None = 520
    Local = 521
    Program = 522
    Rand = 523
    Randc = 524
    Ref = 525
    Solve = 526
    String = 527
    Super = 528
    This = 529
    Throughout = 530
    Var = 531
    Virtual = 532
    Wait_Order = 533
    Last_SV3_1 = 533
    First_SV3_1a = 534
    Covergroup = 534
    Coverpoint = 535
    Endgroup = 536
    Endpackage = 537
    Expect = 538
    Foreach = 539
    Ignore_Bins = 540
    Illegal_Bins = 541
    Matches = 542
    Randcase = 543
    Randsequence = 544
    Tagged = 545
    Wildcard = 546
    Last_SV3_1a = 546
    First_SV2009 = 547
    Implies = 547
    S_Until = 548
    S_Until_With = 549
    Until_With = 550
    Last_SV2009 = 550
    First_Operator = 551
    Op_Equality = 551
    Op_Inequality = 552
    Op_Less = 553
    Op_Less_Equal = 554
    Op_Greater = 555
    Op_Greater_Equal = 556
    Op_Plus = 557
    Op_Minus = 558
    Op_Mul = 559
    Op_Div = 560
    Op_Exp = 561
    Op_Concatenation = 562
    Op_Condition = 563
    Op_Match_Equality = 564
    Op_Match_Inequality = 565
    Op_Match_Less = 566
    Op_Match_Less_Equal = 567
    Op_Match_Greater = 568
    Op_Match_Greater_Equal = 569
    Last_Operator = 569
    First_Attribute = 570
    Base = 570
    Left = 571
    Right = 572
    High = 573
    Low = 574
    Pos = 575
    Val = 576
    Succ = 577
    Pred = 578
    Leftof = 579
    Rightof = 580
    Reverse_Range = 581
    Length = 582
    Delayed = 583
    Stable = 584
    Quiet = 585
    Transaction = 586
    Event = 587
    Active = 588
    Last_Event = 589
    Last_Active = 590
    Last_Value = 591
    Last_Attribute = 591
    First_Vhdl87_Attribute = 592
    Behavior = 592
    Structure = 593
    Last_Vhdl87_Attribute = 593
    First_Vhdl93_Attribute = 594
    Ascending = 594
    Image = 595
    Value = 596
    Driving = 597
    Driving_Value = 598
    Simple_Name = 599
    Instance_Name = 600
    Path_Name = 601
    Last_Vhdl93_Attribute = 601
    First_Vhdl08_Attribute = 602
    Element = 602
    Last_Vhdl08_Attribute = 602
    First_AMS_Attribute = 603
    Contribution = 603
    Dot = 604
    Integ = 605
    Above = 606
    Zoh = 607
    Ltf = 608
    Ztf = 609
    Ramp = 610
    Slew = 611
    Last_AMS_Attribute = 611
    First_Standard = 612
    Std = 612
    Standard = 613
    Boolean = 614
    NFalse = 615
    NTrue = 616
    Character = 617
    Severity_Level = 618
    Note = 619
    Warning = 620
    Error = 621
    Failure = 622
    Universal_Integer = 623
    Universal_Real = 624
    Convertible_Integer = 625
    Convertible_Real = 626
    Integer = 627
    Real = 628
    Time = 629
    Fs = 630
    Ps = 631
    Ns = 632
    Us = 633
    Ms = 634
    Sec = 635
    Min = 636
    Hr = 637
    Max = 638
    Delay_Length = 639
    Now = 640
    Natural = 641
    Positive = 642
    Bit_Vector = 643
    File_Open_Kind = 644
    Read_Mode = 645
    Write_Mode = 646
    Append_Mode = 647
    File_Open_Status = 648
    Open_Ok = 649
    Status_Error = 650
    Name_Error = 651
    Mode_Error = 652
    Foreign = 653
    Boolean_Vector = 654
    To_Bstring = 655
    To_Binary_String = 656
    To_Ostring = 657
    To_Octal_String = 658
    To_Hstring = 659
    To_Hex_String = 660
    Integer_Vector = 661
    Real_Vector = 662
    Time_Vector = 663
    Digits = 664
    Format = 665
    Unit = 666
    Domain_Type = 667
    Quiescent_Domain = 668
    Time_Domain = 669
    Frequency_Domain = 670
    Domain = 671
    Frequency = 672
    Last_Standard = 672
    First_Charname = 673
    Nul = 673
    Soh = 674
    Stx = 675
    Etx = 676
    Eot = 677
    Enq = 678
    Ack = 679
    Bel = 680
    Bs = 681
    Ht = 682
    Lf = 683
    Vt = 684
    Ff = 685
    Cr = 686
    So = 687
    Si = 688
    Dle = 689
    Dc1 = 690
    Dc2 = 691
    Dc3 = 692
    Dc4 = 693
    Nak = 694
    Syn = 695
    Etb = 696
    Can = 697
    Em = 698
    Sub = 699
    Esc = 700
    Fsp = 701
    Gsp = 702
    Rsp = 703
    Usp = 704
    Del = 705
    C128 = 706
    C129 = 707
    C130 = 708
    C131 = 709
    C132 = 710
    C133 = 711
    C134 = 712
    C135 = 713
    C136 = 714
    C137 = 715
    C138 = 716
    C139 = 717
    C140 = 718
    C141 = 719
    C142 = 720
    C143 = 721
    C144 = 722
    C145 = 723
    C146 = 724
    C147 = 725
    C148 = 726
    C149 = 727
    C150 = 728
    C151 = 729
    C152 = 730
    C153 = 731
    C154 = 732
    C155 = 733
    C156 = 734
    C157 = 735
    C158 = 736
    C159 = 737
    Last_Charname = 737
    First_Misc = 738
    Guard = 738
    Deallocate = 739
    File_Open = 740
    File_Close = 741
    Read = 742
    Write = 743
    Flush = 744
    Endfile = 745
    I = 746
    J = 747
    F = 748
    L = 749
    P = 750
    R = 751
    S = 752
    V = 753
    External_Name = 754
    Open_Kind = 755
    First = 756
    Last = 757
    Textio = 758
    Work = 759
    Text = 760
    To_String = 761
    Minimum = 762
    Maximum = 763
    Untruncated_Text_Read = 764
    Textio_Read_Real = 765
    Textio_Write_Real = 766
    Get_Resolution_Limit = 767
    Control_Simulation = 768
    Step = 769
    Index = 770
    Item = 771
    Uu_File_Uu = 772
    Uu_Line_Uu = 773
    Label_Applies_To = 774
    Return_Port_Name = 775
    Map_To_Operator = 776
    Type_Function = 777
    Built_In = 778
    NNone = 779
    Last_Misc = 779
    First_Ieee_Pkg = 780
    Ieee = 780
    Std_Logic_1164 = 781
    VITAL_Timing = 782
    Numeric_Std = 783
    Numeric_Bit = 784
    Numeric_Std_Unsigned = 785
    Std_Logic_Arith = 786
    Std_Logic_Signed = 787
    Std_Logic_Unsigned = 788
    Std_Logic_Textio = 789
    Std_Logic_Misc = 790
    Math_Real = 791
    Last_Ieee_Pkg = 791
    First_Ieee_Name = 792
    Std_Ulogic = 792
    Std_Ulogic_Vector = 793
    Std_Logic = 794
    Std_Logic_Vector = 795
    Rising_Edge = 796
    Falling_Edge = 797
    VITAL_Level0 = 798
    VITAL_Level1 = 799
    Unresolved_Unsigned = 800
    Unresolved_Signed = 801
    To_Integer = 802
    To_Unsigned = 803
    To_Signed = 804
    Resize = 805
    Std_Match = 806
    Shift_Left = 807
    Shift_Right = 808
    Rotate_Left = 809
    Rotate_Right = 810
    To_Bit = 811
    To_Bitvector = 812
    To_Stdulogic = 813
    To_Stdlogicvector = 814
    To_Stdulogicvector = 815
    Is_X = 816
    To_01 = 817
    To_X01 = 818
    To_X01Z = 819
    To_UX01 = 820
    Conv_Signed = 821
    Conv_Unsigned = 822
    Conv_Integer = 823
    Conv_Std_Logic_Vector = 824
    And_Reduce = 825
    Nand_Reduce = 826
    Or_Reduce = 827
    Nor_Reduce = 828
    Xor_Reduce = 829
    Xnor_Reduce = 830
    Ceil = 831
    Floor = 832
    Round = 833
    Log2 = 834
    Sin = 835
    Cos = 836
    Shl = 837
    Shr = 838
    Ext = 839
    Sxt = 840
    Find_Leftmost = 841
    Find_Rightmost = 842
    Last_Ieee_Name = 842
    First_Synthesis = 843
    Allconst = 843
    Allseq = 844
    Anyconst = 845
    Anyseq = 846
    Gclk = 847
    Last_Synthesis = 847
    First_Directive = 848
    Define = 848
    Endif = 849
    Ifdef = 850
    Ifndef = 851
    Include = 852
    Timescale = 853
    Undef = 854
    Protect = 855
    Begin_Protected = 856
    End_Protected = 857
    Key_Block = 858
    Data_Block = 859
    Line = 860
    Celldefine = 861
    Endcelldefine = 862
    Default_Nettype = 863
    Resetall = 864
    Last_Directive = 864
    First_Systask = 865
    Bits = 865
    D_Root = 866
    D_Unit = 867
    Last_Systask = 867
    First_SV_Method = 868
    Size = 868
    Insert = 869
    Delete = 870
    Pop_Front = 871
    Pop_Back = 872
    Push_Front = 873
    Push_Back = 874
    Name = 875
    Len = 876
    Substr = 877
    Exists = 878
    Atoi = 879
    Itoa = 880
    Find = 881
    Find_Index = 882
    Find_First = 883
    Find_First_Index = 884
    Find_Last = 885
    Find_Last_Index = 886
    Num = 887
    Randomize = 888
    Pre_Randomize = 889
    Post_Randomize = 890
    Srandom = 891
    Get_Randstate = 892
    Set_Randstate = 893
    Seed = 894
    State = 895
    Last_SV_Method = 895
    First_BSV = 896
    uAction = 896
    uActionValue = 897
    BVI = 898
    uC = 899
    uCF = 900
    uE = 901
    uSB = 902
    uSBR = 903
    Action = 904
    Endaction = 905
    Actionvalue = 906
    Endactionvalue = 907
    Ancestor = 908
    Clocked_By = 909
    Default_Clock = 910
    Default_Reset = 911
    Dependencies = 912
    Deriving = 913
    Determines = 914
    Enable = 915
    Ifc_Inout = 916
    Input_Clock = 917
    Input_Reset = 918
    Instance = 919
    Endinstance = 920
    Let = 921
    Match = 922
    Method = 923
    Endmethod = 924
    Numeric = 925
    Output_Clock = 926
    Output_Reset = 927
    Par = 928
    Endpar = 929
    Path = 930
    Provisos = 931
    Ready = 932
    Reset_By = 933
    Rule = 934
    Endrule = 935
    Rules = 936
    Endrules = 937
    Same_Family = 938
    Schedule = 939
    Seq = 940
    Endseq = 941
    Typeclass = 942
    Endtypeclass = 943
    Valueof = 944
    uValueof = 945
    Last_BSV = 945
    First_Comment = 946
    Psl = 946
    Pragma = 947
    Synthesis = 948
    Synopsys = 949
    Translate_Off = 950
    Translate_On = 951
    Translate = 952
    Synthesis_Off = 953
    Synthesis_On = 954
    Off = 955
    Last_Comment = 955
    First_PSL = 956
    A = 956
    Af = 957
    Ag = 958
    Ax = 959
    Abort = 960
    Assume_Guarantee = 961
    Before = 962
    Clock = 963
    E = 964
    Ef = 965
    Eg = 966
    Ex = 967
    Endpoint = 968
    Eventually = 969
    Fairness = 970
    Fell = 971
    Forall = 972
    G = 973
    Inf = 974
    Inherit = 975
    Never = 976
    Next_A = 977
    Next_E = 978
    Next_Event = 979
    Next_Event_A = 980
    Next_Event_E = 981
    Prev = 982
    Rose = 983
    Strong = 984
    W = 985
    Whilenot = 986
    Within = 987
    X = 988
    Last_PSL = 988
    First_Edif = 989
    Celltype = 999
    View = 1000
    Viewtype = 1001
    Direction = 1002
    Contents = 1003
    Net = 1004
    Viewref = 1005
    Cellref = 1006
    Libraryref = 1007
    Portinstance = 1008
    Joined = 1009
    Portref = 1010
    Instanceref = 1011
    Design = 1012
    Designator = 1013
    Owner = 1014
    Member = 1015
    Number = 1016
    Rename = 1017
    Userdata = 1018
    Last_Edif = 1018
