# Auto generated Python source file from Ada sources
# Call 'make' in 'src/vhdl' to regenerate:
#
from pydecor import export
from pyGHDL.libghdl import libghdl

Get_Start_Location = libghdl.vhdl__elocations__get_start_location
Set_Start_Location = libghdl.vhdl__elocations__set_start_location

Get_Right_Paren_Location = libghdl.vhdl__elocations__get_right_paren_location
Set_Right_Paren_Location = libghdl.vhdl__elocations__set_right_paren_location

Get_End_Location = libghdl.vhdl__elocations__get_end_location
Set_End_Location = libghdl.vhdl__elocations__set_end_location

Get_Is_Location = libghdl.vhdl__elocations__get_is_location
Set_Is_Location = libghdl.vhdl__elocations__set_is_location

Get_Begin_Location = libghdl.vhdl__elocations__get_begin_location
Set_Begin_Location = libghdl.vhdl__elocations__set_begin_location

Get_Then_Location = libghdl.vhdl__elocations__get_then_location
Set_Then_Location = libghdl.vhdl__elocations__set_then_location

Get_Use_Location = libghdl.vhdl__elocations__get_use_location
Set_Use_Location = libghdl.vhdl__elocations__set_use_location

Get_Loop_Location = libghdl.vhdl__elocations__get_loop_location
Set_Loop_Location = libghdl.vhdl__elocations__set_loop_location

Get_Generate_Location = libghdl.vhdl__elocations__get_generate_location
Set_Generate_Location = libghdl.vhdl__elocations__set_generate_location

Get_Generic_Location = libghdl.vhdl__elocations__get_generic_location
Set_Generic_Location = libghdl.vhdl__elocations__set_generic_location

Get_Port_Location = libghdl.vhdl__elocations__get_port_location
Set_Port_Location = libghdl.vhdl__elocations__set_port_location

Get_Generic_Map_Location = libghdl.vhdl__elocations__get_generic_map_location
Set_Generic_Map_Location = libghdl.vhdl__elocations__set_generic_map_location

Get_Port_Map_Location = libghdl.vhdl__elocations__get_port_map_location
Set_Port_Map_Location = libghdl.vhdl__elocations__set_port_map_location

Get_Arrow_Location = libghdl.vhdl__elocations__get_arrow_location
Set_Arrow_Location = libghdl.vhdl__elocations__set_arrow_location

Get_Colon_Location = libghdl.vhdl__elocations__get_colon_location
Set_Colon_Location = libghdl.vhdl__elocations__set_colon_location

Get_Assign_Location = libghdl.vhdl__elocations__get_assign_location
Set_Assign_Location = libghdl.vhdl__elocations__set_assign_location

